package de.ozgcloud.eingang.fim;

import static org.assertj.core.api.Assertions.*;
import static org.mockito.ArgumentMatchers.*;
import static org.mockito.Mockito.*;

import java.util.Collections;
import java.util.Map;

import org.junit.jupiter.api.BeforeEach;
import org.junit.jupiter.api.DisplayName;
import org.junit.jupiter.api.Nested;
import org.junit.jupiter.api.Test;
import org.mockito.InjectMocks;
import org.mockito.Spy;

import de.ozgcloud.eingang.common.formdata.Antragsteller;
import de.ozgcloud.eingang.common.formdata.FormData;
import de.ozgcloud.eingang.common.formdata.FormDataTestFactory;

class AntragstellerMapperTest {

	@Spy
	@InjectMocks
	private AntragstellerMapper mapper;

	@DisplayName("Parse formData")
	@Nested
	class TestParseFormData {

		private final FormData formData = FormDataTestFactory.createBuilder().antragsteller(null).build();

		@DisplayName("on existing antragsteller")
		@Nested
		class TestOnExistingAntragsteller {

			private final Map<String, Object> antragstellerMap = AntragstellerTestFactory.createAntragstellerAsFimMap();
			private final Antragsteller antragsteller = AntragstellerTestFactory.create();

			@BeforeEach
			void mock() {
				doReturn(antragstellerMap).when(mapper).getAntragsteller(any());
				doReturn(antragsteller).when(mapper).mapAntragsteller(any());
			}

			@Test
			void shouldCallGetAntragsteller() {
				mapper.parseFormData(formData);

				verify(mapper).getAntragsteller(FormDataTestFactory.FORM_DATA);
			}

			@Test
			void shouldCallMapAntragsteller() {
				mapper.parseFormData(formData);

				verify(mapper).mapAntragsteller(antragstellerMap);
			}

			@Test
			void shouldReturnFormDataWithAntragsteller() {
				var mappedFormData = mapper.parseFormData(formData);

				assertThat(mappedFormData.getAntragsteller()).isNotNull();
			}
		}

		@DisplayName("on missing antragsteller")
		@Nested
		class TestOnMissingAntragsteller {

			@BeforeEach
			void mock() {
				doReturn(Collections.emptyMap()).when(mapper).getAntragsteller(any());
			}

			@Test
			void shouldReturnNullAsAntragsteller() {
				var mappedFormData = mapper.parseFormData(formData);

				assertThat(mappedFormData.getAntragsteller()).isNull();
			}
		}
	}

	@DisplayName("Get antragsteller")
	@Nested
	class TestGetAntragsteller {

		@Test
		void shouldReturnMapIfAntragstellerExists() {
			var antragstellerMap = AntragstellerTestFactory.createAntragstellerAsFimMap();
			var formDataMap = Map.<String, Object>of(AntragstellerMapper.ANTRAGSTELLER_KEY, Map.of(FimDataUtil.VALUE_KEY, antragstellerMap));

			var antragsteller = mapper.getAntragsteller(formDataMap);

			assertThat(antragsteller).isNotEmpty().usingRecursiveComparison().isEqualTo(antragstellerMap);
		}

		@Test
		void shouldReturnEmptyMapIfAntragstellerIsMissing() {
			var antragsteller = mapper.getAntragsteller(Collections.emptyMap());

			assertThat(antragsteller).isNull();
		}
	}

	@DisplayName("Map antragsteller")
	@Nested
	class TestMapAntragsteller {

		@Test
		void shouldMap() {
			var antragstellerMap = AntragstellerTestFactory.createAntragstellerAsFimMap();

			var antragsteller = mapper.mapAntragsteller(antragstellerMap);

			assertThat(antragsteller).usingRecursiveComparison().isEqualTo(AntragstellerTestFactory.create());
		}

		@Test
		void shouldMapEmpty() {
			var formData = Collections.<String, Object>emptyMap();

			var antragsteller = mapper.mapAntragsteller(formData);

			assertThat(antragsteller).usingRecursiveComparison().isEqualTo(Antragsteller.builder().build());
		}
	}
}