package de.ozgcloud.eingang.fim;

import static org.assertj.core.api.Assertions.*;

import java.io.File;
import java.util.List;
import java.util.Map;
import java.util.Optional;

import org.junit.jupiter.api.DisplayName;
import org.junit.jupiter.api.Nested;
import org.junit.jupiter.api.Test;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.autoconfigure.ImportAutoConfiguration;
import org.springframework.boot.test.context.SpringBootTest;
import org.springframework.test.context.ActiveProfiles;

import de.ozgcloud.common.test.ITCase;
import de.ozgcloud.eingang.common.formdata.Antragsteller;
import de.ozgcloud.eingang.common.formdata.FormData;
import de.ozgcloud.eingang.common.formdata.FormData.FormDataControl;
import de.ozgcloud.eingang.common.formdata.FormData.Representations;
import de.ozgcloud.eingang.common.formdata.FormHeader;
import de.ozgcloud.eingang.common.formdata.IncomingFile;
import de.ozgcloud.eingang.common.formdata.IncomingFileTestFactory;
import lombok.SneakyThrows;

@ITCase
@ActiveProfiles({ "itcase", "test" })
@ImportAutoConfiguration
@SpringBootTest(classes = FimTestConfig.class)
class FimBasedAdapterITCase {

	@Autowired // NOSONAR
	private FimBasedAdapter fimBasedAdapter;

	@Nested
	class TestFallback {

		@Test
		void shouldWithFallbackUnknownScheme() {
			var incomingFile = IncomingFile.builder().name("src/test/resources/test3/Antrag.xml")
					.file(new File("src/test/resources/test3/Antrag.xml"))
					.build();
			var incomingFile2 = IncomingFile.builder().name("src/test/resources/test3/fim_xtaMetadata.xml").build();
			var initialFormData = FormData.builder()
					.header(FormHeader.builder().channel("XTA").formEngineName("FIM").build())
					.representations(List.of(incomingFile, incomingFile2)).build();

			var formData = fimBasedAdapter.parseFormData(initialFormData);

			assertThat(formData).isNotNull();
			assertThat(formData.getFormData()).isEqualTo(getParsedDocumentMap());
		}

		private Map<String, Object> getParsedDocumentMap() {
			return Map.of(
					"G17003529", Map.of(
							"label", "G17003529",
							"value", Map.of(
									"G05001479", Map.of(
											"label", "G05001479",
											"value", Map.of(
													"G05001480", Map.of(
															"label", "G05001480",
															"value", Map.of(
																	"F05002750",
																	Map.of("label", "F05002750", "value", "d447e43a-5723-4821-a170-cb44d2dbf143"),
																	"F05002751", Map.of("label", "F05002751", "value", "2022-08-15T09:30:47"),
																	"F05002752",
																	Map.of("label", "F05002752", "value", "fim.S17000652.17000652001004"),
																	"F05002753",
																	Map.of("label", "F05002753", "value", "urn:fim:Versammlungsanzeige:1.4"))),
													"G05001481", Map.of(
															"label", "G05001481",
															"value", Map.of(
																	"F05002754", Map.of("label", "F05002754", "value", "Celle"),
																	"F05002755", Map.of("label", "F05002755", "value", "vbe:010550120100"),
																	"F05002756", Map.of("label", "F05002756", "value", "Versammlungsbehörde"))),
													"G05001482", Map.of(
															"label", "G05001482",
															"value", Map.of(
																	"F05002754",
																	Map.of("label", "F05002754", "value",
																			"OSI-Onlinedienst Niedersachsen Versammlungsanzeige"),
																	"F05002755", Map.of("label", "F05002755", "value", "vbe:010550120100"),
																	"F05002756",
																	Map.of("label", "F05002756", "value", "Engagement- und Hobbyportal"))))),
									"F17005454", Map.of("label", "F17005454", "value", "true"),
									"F17005455", Map.of("label", "F17005455", "value", "true"),
									"F17005533", Map.of("label", "F17005533", "value", "String"))),
					"F17009191", Map.of("label", "F17009191", "value", "true"),
					"F17003371", Map.of("label", "F17003371", "value", "String"));
		}
	}

	@DisplayName("Parse formData")
	@Nested
	class TestParseFormData {

		private final String fileName = "antrag.xml";
		private final FormData emptyFormData = FormData.builder()
				.control(FormDataControl.builder()
						.representations(Optional.of(Representations.builder()
								.primaryFormDataRepresentation(fileName)
								.build()))
						.build())
				.representation(IncomingFile.builder()
						.name(fileName)
						.file(new File("src/test/resources/" + fileName))
						.build())
				.build();

		@Test
		void shouldHaveHeaderFormName() {
			var formData = fimBasedAdapter.parseFormData(emptyFormData);

			assertThat(formData).isNotNull();
			assertThat(formData.getHeader().getFormName()).isEqualTo("urn:fim:Versammlungsanzeige:1.4");
		}

		@Test
		void shouldHaveAntragsteller() {
			var formData = fimBasedAdapter.parseFormData(emptyFormData);

			assertThat(formData).isNotNull();
			assertThat(formData.getAntragsteller()).usingRecursiveComparison().isEqualTo(createExpectedAntragsteller());
		}

		private Antragsteller createExpectedAntragsteller() {
			return Antragsteller.builder()
					.vorname("Jörg").nachname("Smith")
					.email("example@email.de")
					.strasse("Muster")
					.hausnummer("1")
					.plz("12345")
					.ort("Muster")
					.build();
		}
	}

	@DisplayName("Process data")
	@Nested
	class TestDoParse {

		@SneakyThrows
		@Test
		void shouldTransformSimpleDocument() {
			var file = IncomingFileTestFactory.createBuilder().file(new File("src/test/resources/test2.xml")).build();

			var formData = fimBasedAdapter.doParse(file);

			assertThat(formData).isEqualTo(FimDocumentTestHelper.getTest2AsMap());
		}

		@SneakyThrows
		@Test
		void shouldTransformSimpleDocumentWithoutNamespace() {
			var file = IncomingFileTestFactory.createBuilder().file(new File("src/test/resources/test3.xml")).build();

			var formData = fimBasedAdapter.doParse(file);

			assertThat(formData).isEqualTo(FimDocumentTestHelper.getTest3AsMap());
		}

		@SneakyThrows
		@Test
		void shouldTransformDocument() {
			var file = IncomingFileTestFactory.createBuilder().file(new File("src/test/resources/S17000652V1.4_test01.xml")).build();

			var formData = fimBasedAdapter.doParse(file);

			assertThat(formData).isNotEmpty().isEqualTo(FimDocumentTestHelper.getVersammlungsAnzeigeAsMap());
		}
	}
}